import _isPlainObject from "lodash/isPlainObject";
import _isEmpty from "lodash/isEmpty";
var _excluded = ["className", "placeholderClass", "placeholderClassName", "value", "readonly", "disabled", "align", "color", "clearable", "clearTrigger", "clearIcon", "onInput", "onChange", "onFocus", "onBlur", "onClear"];

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { useUncontrolled } from "@nickeylin/taroify-hooks";
import { Clear } from "@nickeylin/taroify-icons";
import { cloneIconElement } from "@nickeylin/taroify-icons/utils";
import classNames from "classnames";
import * as React from "react";
import { useMemo, useState } from "react";
import { prefixClassname } from "../styles";
import { preventDefault } from "../utils/dom/event";
import NativeInput from "./native-input";
export function resolveOnChange(e, onChange, detailValue) {
  if (!onChange) {
    return;
  }

  if (e.type === "click" || e.type === "tap") {
    var {
      detail
    } = e;
    var event = Object.assign({}, e, {
      detail: _objectSpread(_objectSpread({}, _isPlainObject(detail) ? detail : {}), {}, {
        value: ""
      })
    });
    onChange(event);
    return;
  }

  if (detailValue !== undefined) {
    var {
      detail: _detail
    } = e;

    var _event = Object.assign({}, e, {
      detail: _objectSpread(_objectSpread({}, _isPlainObject(_detail) ? _detail : {}), {}, {
        value: detailValue
      })
    });

    onChange(_event);
    return;
  }

  onChange(e);
}

function Input(props) {
  var {
    className,
    placeholderClass,
    placeholderClassName,
    value: valueProp,
    readonly,
    disabled,
    align,
    color,
    clearable,
    clearTrigger = "focus",
    clearIcon = /*#__PURE__*/React.createElement(Clear, null),
    onInput,
    onChange,
    onFocus,
    onBlur,
    onClear
  } = props,
      restProps = _objectWithoutProperties(props, _excluded);

  var {
    value,
    setValue
  } = useUncontrolled({
    value: valueProp
  });
  var [focused, setFocused] = useState(false);
  var allowClear = useMemo(() => {
    if (clearable && !disabled) {
      var hasValue = !_isEmpty(value);
      var trigger = clearTrigger === "always" || clearTrigger === "focus" && focused;
      return hasValue && trigger;
    }

    return false;
  }, [clearTrigger, clearable, disabled, focused, value]);

  var handleClear = event => {
    preventDefault(event, true);
    resolveOnChange(event, onChange, "");
    resolveOnChange(event, onInput, "");
    onClear === null || onClear === void 0 ? void 0 : onClear(event);
    setValue("");
  };

  var handleFocus = event => {
    setFocused(true);
    onFocus === null || onFocus === void 0 ? void 0 : onFocus(event);
  };

  var handleInput = event => {
    onInput === null || onInput === void 0 ? void 0 : onInput(event);
    onChange === null || onChange === void 0 ? void 0 : onChange(event);
    setValue(event.detail.value);
  };

  var handleBlur = event => {
    onBlur === null || onBlur === void 0 ? void 0 : onBlur(event);
    setTimeout(() => setFocused(false), 80);
  };

  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(NativeInput, _objectSpread({
    className: classNames(prefixClassname("input"), {
      [prefixClassname("input--disabled")]: disabled,
      [prefixClassname("input--readonly")]: readonly,
      [prefixClassname("input--right")]: align === "right",
      [prefixClassname("input--center")]: align === "center",
      [prefixClassname("input--left")]: align === "left",
      [prefixClassname("input--primary")]: color === "primary",
      [prefixClassname("input--info")]: color === "info",
      [prefixClassname("input--success")]: color === "success",
      [prefixClassname("input--warning")]: color === "warning",
      [prefixClassname("input--danger")]: color === "danger"
    }, className),
    placeholderClass: classNames(placeholderClass, placeholderClassName, prefixClassname("input__placeholder"), {
      [prefixClassname("input__placeholder--readonly")]: readonly,
      [prefixClassname("input__placeholder--primary")]: color === "primary",
      [prefixClassname("input__placeholder--info")]: color === "info",
      [prefixClassname("input__placeholder--success")]: color === "success",
      [prefixClassname("input__placeholder--warning")]: color === "warning",
      [prefixClassname("input__placeholder--danger")]: color === "danger"
    }),
    disabled: disabled || readonly,
    readonly: readonly,
    color: color,
    value: value,
    onFocus: handleFocus,
    onInput: handleInput,
    onBlur: handleBlur
  }, restProps)), allowClear && cloneIconElement(clearIcon, {
    className: prefixClassname("input__clear"),
    onClick: handleClear
  }));
}

export default Input;
//# sourceMappingURL=input.js.map
