import _isBoolean from "lodash/isBoolean";
var _excluded = ["className", "value", "length", "gutter", "mask", "focused", "focus", "feedback", "onFocus"];

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import { View } from "@tarojs/components";
import classNames from "classnames";
import * as React from "react";
import { useCallback, useMemo } from "react";
import { prefixClassname } from "../styles";
import { HAIRLINE_BORDER_LEFT, HAIRLINE_BORDER_SURROUND } from "../styles/hairline";
import { stopPropagation } from "../utils/dom/event";
import { createVariantElement } from "../utils/element";
import { addUnitPx } from "../utils/format/unit";
import PasswordInputFeedback from "./password-input-feedback";

function PasswordInput(props) {
  var {
    className,
    value = "",
    length = 6,
    gutter,
    mask = true,
    focused: focusedProp,
    focus: focusProp = false,
    feedback: feedbackProp,
    onFocus
  } = props,
      restProps = _objectWithoutProperties(props, _excluded);

  var feedback = createVariantElement(PasswordInputFeedback, feedbackProp);
  var focus = useMemo(() => _isBoolean(focusedProp) ? focusedProp : focusProp, [focusProp, focusedProp]);

  if (_isBoolean(focusedProp)) {
    console.warn("[Deprecated] The focused prop is deprecated. Please use the focus prop.");
  }

  var onTouchStart = useCallback(event => {
    stopPropagation(event);
    onFocus === null || onFocus === void 0 ? void 0 : onFocus(event);
  }, [onFocus]);
  var points = useMemo(() => {
    var Points = [];

    for (var i = 0; i < length; i++) {
      var char = value[i];
      var bordered = i !== 0 && !gutter;
      var showCursor = focus && i === value.length;
      var style = void 0;

      if (i !== 0 && gutter) {
        style = {
          marginLeft: addUnitPx(gutter)
        };
      }

      Points.push( /*#__PURE__*/React.createElement(View, {
        key: i,
        className: classNames(prefixClassname("password-input__item"), {
          [HAIRLINE_BORDER_LEFT]: bordered,
          [prefixClassname("password-input__item--focus")]: showCursor
        }),
        style: style
      }, mask ? /*#__PURE__*/React.createElement(View, {
        className: prefixClassname("password-input__item--mask"),
        style: {
          visibility: char ? "visible" : "hidden"
        }
      }) : char, showCursor && /*#__PURE__*/React.createElement(View, {
        className: prefixClassname("password-input__cursor")
      })));
    }

    return Points;
  }, [focus, gutter, length, mask, value]);
  return /*#__PURE__*/React.createElement(View, _objectSpread({
    className: classNames(prefixClassname("password-input"), className)
  }, restProps), /*#__PURE__*/React.createElement(View, {
    className: classNames(prefixClassname("password-input__security"), {
      [HAIRLINE_BORDER_SURROUND]: !gutter
    }),
    onTouchStart: onTouchStart,
    children: points
  }), feedback);
}

export default PasswordInput;
//# sourceMappingURL=password-input.js.map
