import _isUndefined from "lodash/isUndefined";
import _isNumber from "lodash/isNumber";
var _excluded = ["className", "style", "defaultOpen", "open", "placement", "rounded", "children", "duration", "transaction", "transactionTimeout", "transitionAppear", "mountOnEnter", "onClose", "onTransitionEnter", "onTransitionEntered", "onTransitionExited"];

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import { useUncontrolled } from "@nickeylin/taroify-hooks";
import { View } from "@tarojs/components";
import classNames from "classnames";
import * as React from "react";
import { Children, forwardRef, isValidElement, useMemo } from "react";
import Backdrop from "../backdrop";
import { prefixClassname } from "../styles";
import Transition, { TransitionName } from "../transition";
import { isElementOf } from "../utils/validate";
import PopupBackdrop from "./popup-backdrop";
import PopupClose from "./popup-close";
import PopupContext from "./popup.context";

function toTransactionName(placement) {
  if (placement === "top") {
    return TransitionName.SlideDown;
  }

  if (placement === "bottom") {
    return TransitionName.SlideUp;
  }

  if (placement === "right") {
    return TransitionName.SlideRight;
  }

  if (placement === "left") {
    return TransitionName.SlideLeft;
  }

  return TransitionName.Fade;
}

function usePopupChildren(children) {
  return useMemo(() => {
    var __children__ = {
      backdrop: undefined,
      close: undefined,
      content: []
    };
    Children.forEach(children, child => {
      if ( /*#__PURE__*/isValidElement(child)) {
        var element = child;

        if (isElementOf(element, Backdrop)) {
          __children__.backdrop = element;
        } else if (element.type === PopupClose) {
          __children__.close = element;
        } else {
          __children__.content.push(child);
        }
      } else {
        __children__.content.push(child);
      }
    });
    return __children__;
  }, [children]);
}

var Popup = /*#__PURE__*/forwardRef((props, ref) => {
  var {
    className,
    style: styleProp,
    defaultOpen,
    open: openProp,
    placement,
    rounded = false,
    children,
    duration,
    transaction,
    transactionTimeout,
    transitionAppear = true,
    mountOnEnter = true,
    onClose,
    onTransitionEnter,
    onTransitionEntered,
    onTransitionExited
  } = props,
      restProps = _objectWithoutProperties(props, _excluded);

  var {
    value: open
  } = useUncontrolled({
    defaultValue: defaultOpen,
    value: openProp
  });
  var transactionName = transaction !== null && transaction !== void 0 ? transaction : toTransactionName(placement);
  var {
    backdrop = /*#__PURE__*/React.createElement(PopupBackdrop, null),
    close,
    content
  } = usePopupChildren(children);
  var durationStyle = useMemo(() => _isNumber(duration) ? {
    "--animation-duration-base": "".concat(duration, "ms")
  } : {}, [duration]);
  return /*#__PURE__*/React.createElement(PopupContext.Provider, {
    value: {
      open,
      duration,
      placement,
      onClose
    }
  }, /*#__PURE__*/React.createElement(Transition, {
    in: open,
    name: transactionName,
    appear: transitionAppear,
    timeout: transactionTimeout,
    mountOnEnter: mountOnEnter,
    onEnter: onTransitionEnter,
    onEntered: onTransitionEntered,
    onExited: onTransitionExited
  }, /*#__PURE__*/React.createElement(View, _objectSpread({
    ref: ref,
    className: classNames(prefixClassname("popup"), {
      [prefixClassname("popup--rounded")]: rounded,
      [prefixClassname("popup--center")]: _isUndefined(placement),
      [prefixClassname("popup--top")]: placement === "top",
      [prefixClassname("popup--right")]: placement === "right",
      [prefixClassname("popup--bottom")]: placement === "bottom",
      [prefixClassname("popup--left")]: placement === "left"
    }, className),
    style: _objectSpread(_objectSpread({}, durationStyle), styleProp)
  }, restProps), close, content)), backdrop);
});
export default Popup;
//# sourceMappingURL=popup.js.map
