var _excluded = ["className", "style", "type", "disabled", "onClick", "onTouchStart", "onTouchEnd", "onTouchCancel"];

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import { View } from "@tarojs/components";
import classNames from "classnames";
import * as React from "react";
import { useCallback, useContext, useRef } from "react";
import { prefixClassname } from "../styles";
import { preventDefault } from "../utils/dom/event";
import { getSizeStyle } from "../utils/format/unit";
import StepperContext from "./stepper.context";
var LONG_PRESS_INTERVAL = 200;
var LONG_PRESS_START_TIME = 600;

function StepperButton(props) {
  var {
    className,
    style,
    type = "decrease",
    disabled: disabledProp,
    onClick: _onClick,
    onTouchStart: _onTouchStart,
    onTouchEnd: _onTouchEnd,
    onTouchCancel: _onTouchCancel
  } = props,
      restProps = _objectWithoutProperties(props, _excluded);

  var {
    value = 0,
    min = 0,
    max = 0,
    size,
    disabled: disabledCtx,
    longPress,
    onStep
  } = useContext(StepperContext);
  var disabled = disabledProp || disabledCtx || type === "decrease" && value <= min || type === "increase" && value >= max;
  var longPressRef = useRef(false);
  var longPressTimerRef = useRef();
  var longPressStep = useCallback(() => {
    longPressTimerRef.current = setTimeout(() => {
      onStep === null || onStep === void 0 ? void 0 : onStep(type);
      longPressStep();
    }, LONG_PRESS_INTERVAL);
  }, [onStep, type]);
  var handleTouchStart = useCallback(() => {
    if (longPress) {
      longPressRef.current = false;

      if (longPressTimerRef.current) {
        clearTimeout(longPressTimerRef.current);
      }

      longPressTimerRef.current = setTimeout(() => {
        longPressRef.current = true;
        onStep === null || onStep === void 0 ? void 0 : onStep(type);
        longPressStep();
      }, LONG_PRESS_START_TIME);
    }
  }, [longPress, longPressStep, onStep, type]);
  var handleTouchEnd = useCallback(event => {
    if (longPress) {
      if (longPressTimerRef.current) {
        clearTimeout(longPressTimerRef.current);
      }

      if (longPressRef.current) {
        preventDefault(event);
      }
    }
  }, [longPress]);
  return /*#__PURE__*/React.createElement(View, _objectSpread({
    className: classNames(prefixClassname("stepper__".concat(type)), {
      [prefixClassname("stepper__".concat(type, "--disabled"))]: disabled
    }, className),
    style: getSizeStyle(size),
    onClick: event => {
      preventDefault(event);
      _onClick === null || _onClick === void 0 ? void 0 : _onClick(event);

      if (!disabled) {
        onStep === null || onStep === void 0 ? void 0 : onStep(type);
      }
    },
    onTouchStart: event => {
      _onTouchStart === null || _onTouchStart === void 0 ? void 0 : _onTouchStart(event);

      if (!disabled) {
        handleTouchStart();
      }
    },
    onTouchEnd: event => {
      _onTouchEnd === null || _onTouchEnd === void 0 ? void 0 : _onTouchEnd(event);
      handleTouchEnd(event);
    },
    onTouchCancel: event => {
      _onTouchCancel === null || _onTouchCancel === void 0 ? void 0 : _onTouchCancel(event);
      handleTouchEnd(event);
    }
  }, restProps));
}

export default StepperButton;
//# sourceMappingURL=stepper-button.js.map
