import _size from "lodash/size";
import _isBoolean from "lodash/isBoolean";
var _excluded = ["value"],
    _excluded2 = ["className", "defaultValue", "value", "duration", "lazyRender", "animated", "swipeable", "sticky", "theme", "ellipsis", "bordered", "children", "onTabClick", "onChange", "onScroll"];

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import { useUncontrolled } from "@nickeylin/taroify-hooks";
import { View } from "@tarojs/components";
import classNames from "classnames";
import * as React from "react";
import { Children, isValidElement, useCallback, useMemo, useRef } from "react";
import Sticky from "../sticky";
import { prefixClassname } from "../styles";
import TabPane from "./tab-pane";
import { TabsContent } from "./tabs-content";
import TabsHeader from "./tabs-header";
import TabsContext from "./tabs.context";

function useTabObjects(children) {
  return useMemo(() => {
    var tabObjects = [];
    Children.forEach(children, node => {
      if (! /*#__PURE__*/isValidElement(node)) {
        return node;
      }

      var element = node;

      if (element.type !== TabPane) {
        return element;
      }

      var {
        key,
        props
      } = element;

      var index = _size(tabObjects);

      var {
        value
      } = props,
          restProps = _objectWithoutProperties(props, _excluded);

      tabObjects.push(_objectSpread({
        key: key !== null && key !== void 0 ? key : index,
        index,
        value: value !== null && value !== void 0 ? value : index
      }, restProps));
    });
    return tabObjects;
  }, [children]);
}

function useTabsSticky(sticky) {
  if (sticky === false) {
    return undefined;
  }

  if (_isBoolean(sticky) && sticky) {
    return {
      offsetTop: 0
    };
  }

  return sticky;
}

function Tabs(props) {
  var {
    className,
    defaultValue,
    value: valueProp,
    duration = 300,
    lazyRender = true,
    animated = false,
    swipeable = false,
    sticky = false,
    theme = "line",
    ellipsis = true,
    bordered,
    children: childrenProp,
    onTabClick,
    onChange,
    onScroll
  } = props,
      restProps = _objectWithoutProperties(props, _excluded2);

  var {
    value = 0,
    getValue,
    setValue
  } = useUncontrolled({
    defaultValue,
    value: valueProp
  });
  var rootRef = useRef();
  var stickyProps = useTabsSticky(sticky);
  var tabObjects = useTabObjects(childrenProp);
  var index = useMemo(() => {
    for (var tab of tabObjects) {
      if (tab.value === value) {
        return tab.index;
      }
    }
  }, [tabObjects, value]);
  var onTabChange = useCallback(event => {
    if (!event.disabled) {
      if (getValue() !== event.value) {
        onChange === null || onChange === void 0 ? void 0 : onChange(event.value, event);
        setValue(event.value);
      }
    }
  }, [getValue, onChange, setValue]);
  var handleTabClick = useCallback(event => {
    onTabClick === null || onTabClick === void 0 ? void 0 : onTabClick(event);
    onTabChange(event);
  }, [onTabChange, onTabClick]);
  var headerRender = useMemo(() => /*#__PURE__*/React.createElement(TabsHeader, {
    value: value,
    theme: theme,
    bordered: bordered,
    ellipsis: ellipsis,
    tabObjects: tabObjects,
    onTabClick: handleTabClick
  }), [bordered, ellipsis, handleTabClick, tabObjects, theme, value]);
  return /*#__PURE__*/React.createElement(TabsContext.Provider, {
    value: {
      index,
      value,
      duration,
      lazyRender,
      animated,
      swipeable,
      theme,
      ellipsis,
      bordered,
      tabObjects
    }
  }, /*#__PURE__*/React.createElement(View, _objectSpread({
    ref: rootRef,
    className: classNames(prefixClassname("tabs"), {
      [prefixClassname("tabs--line")]: theme === "line",
      [prefixClassname("tabs--card")]: theme === "card"
    }, className)
  }, restProps), stickyProps ? /*#__PURE__*/React.createElement(Sticky, {
    container: rootRef,
    offsetTop: stickyProps.offsetTop,
    children: headerRender,
    onScroll: onScroll
  }) : headerRender, /*#__PURE__*/React.createElement(TabsContent, {
    value: value,
    lazyRender: lazyRender,
    duration: duration,
    animated: animated,
    swipeable: swipeable,
    tabObjects: tabObjects,
    onTabChange: onTabChange
  })));
}

export default Tabs;
//# sourceMappingURL=tabs.js.map
